//pocket Geiger からのデータを保存し、放射線が照射したとき音が鳴り、赤丸で表示するスケッチ。　　2019.8

// ***** pocket Geigerの接続機器（Micro:bit,Arduino等)のport番号と合わせてください。 *****
// ***** 例：portNo = 0 がportの1番目です。 *****

int portNo = 1; 

import processing.serial.*;
import ddf.minim.*;
Serial arduino;
String datastr;
PrintWriter output;
Minim minim;
AudioSample song;
float[] uSvData;
int count;
int soundNo = 1;
int graphNo = 1;
int t=0;

void setup()
{
  size(465, 383);
  textSize(14);
  println(Serial.list());  // 使えるポート一覧
  arduino = new Serial(this, Serial.list()[portNo], 9600); 
  arduino.clear();
  output = createWriter("PocketGeiger_Data.txt"); //テキストファイル書き出し用
  arduino.bufferUntil( '\n' );
  background(0); 
  fill(255, 241, 15);
  PFont font = createFont("MS Gothic", 22, true);
  textFont(font); 
  textSize(28); 
  text( "PocketGeiger放射線測定モニター", 20, 30);
  fill(255, 255, 255);
  textSize(20); 
  text( "データが入ってくるまで少々お待ち下さい", 40, 140);
  text( "測定値出力間隔（初期値60秒）に依存します", 40, 175);
  textSize(12);
  text( "通信が不成立ならport番号が正しいか確認してください", 40, 250);
  text( "Processing下部黒色欄（コンソール）に赤字で「null」の文字が表示", 40, 280);
  text( "されたら再度「Run」してください", 40, 300);
  
  minim = new Minim( this );
  song = minim.loadSample("sound.mp3");  //効果音
  uSvData = new float[301]; //μをuで表す
  output.println("Date & Time,hour[h],sec[s],放射線検出数,cpm,uSv/h,uSv/hError,測定値出力間隔（秒）" ); //項目名
  output.flush();  // 出力バッファデータ書き出し
  frameRate(10);
}

void draw()
{
  t=t+1;
  if (t==10) {
    strokeWeight( 1 );  //赤丸を１秒間表示後に消す
    stroke(0, 0, 0 );
    fill( 0, 0, 0);
    ellipse( 442, 105, 17, 17 );
    t=0;
  }
}

void serialEvent( Serial arduino ) //データを受信したら作動する
{
  datastr = arduino.readStringUntil( '\n' ); // Arduino からの文字列を読み出す
  String st = trim(datastr);
  if (st.length()==1 )  //一文字を受信したら音を鳴らす
  {
    strokeWeight( 1 );  //赤丸で表示
    stroke(255, 0, 0 );
    fill( 255, 0, 0);
    ellipse( 442, 105, 17, 17 );
    t=0; 

    if (soundNo == 1)
    {
      song.trigger();
    }
  } else  //データを受信したら処理をする
  {
    String datetimestr = nf(year(), 2)+"/"+nf(month(), 2)+"/"+nf(day(), 2)+" "+nf(hour(), 2) + ":" + nf(minute(), 2) + ":" + nf(second(), 2);
    output.print(datetimestr + ", " + datastr );
    background(0);  
    fill(255, 241, 15);
    textSize(28); 
    text( "PocketGeiger放射線測定モニター", 20, 30); 
    fill(255);
    textSize(14); 
    text("Date & Time", 20, 50);
    text("経過時間[hour:sec], 放射線検出数, cpm,", 20, 98); 
    textSize(20); 
    text( datetimestr, 20, 75); 
    String[] s =splitTokens(datastr, ",");
    text( s[0]+" : "+s[1], 50, 123); 
    text( s[2], 196, 123); 
    text( s[3], 250, 123); 
    String Sv = s[4];
    String dSv = trim(s[5]);
    fill(229, 72, 0);
    textSize(27); 
    text("放射線量 "+ Sv + " ± "+ dSv + " [μSv/h]", 20, 159); 
    fill(255);
    textSize(16); 
    text("データは PocketGeiger_Data.txt として保存されます", 20, 185);
    textSize(14); 
    text("測定値の安定まで数分必要", 265, 52);   
    if (soundNo == 1) {
      text("放射線検出時の音：オン", 265, 70);
    } else if (soundNo == 0) {
      text("放射線検出時の音：オフ", 265, 70);
    }  
    text("(Sキーで切替)", 360, 88);
    text("放射線検出表示", 320, 110);
    text("測定値出力間隔(s)", 320, 130);
    text(s[6], 440, 130); 

    output.flush();  // 出力バッファに残っているデータを全て書き出す
    if (count<301) {
      count=count + 1;
    }   

    for (int i=0; i<300; ++i)   //データをずらして格納する
    {
      uSvData[300-i]=uSvData[300-i-1];
    }
    uSvData[0] = float(s[4]);

    if (graphNo == 1) {
      graphA();
    } else if (graphNo == 0) {
      graphB();
    }
  }
}

void graphA()  //横軸１倍表示
{
  graphFrame();
  textSize(14); 
  text("現在", 412, 373);
  text("30回前", 10, 373);
  text("20回前", 134+10, 373);
  text("10回前", 134*2+10, 373);

  strokeWeight( 1 );
  stroke(250, 200, 0 );
  fill( 250, 200, 0);
  if (count<31) {
    for (int i=0; i<count; ++i)
      ellipse( 430-13.6*i, 350-uSvData[i]*937.5, 3, 3 );
  } else {
    for (int i=0; i<31; ++i)
      ellipse( 430-13.6*i, 350-uSvData[i]*937.5, 3, 3 );
  }
}

void graphB()  //横軸10倍表示
{
  graphFrame();
  textSize(14);
  text("現在", 412, 373);
  text("300回前", 10, 373);
  text("200回前", 134+10, 373);
  text("100回前", 134*2+10, 373);

  strokeWeight( 1 );
  stroke(250, 200, 0 );
  fill( 250, 200, 0);
  for (int i=0; i<count; ++i)
  {
    ellipse( 430-1.36*i, 350-uSvData[i]*937.5, 1, 1 );
  }
}

void graphFrame() //グラフ領域の線・文字表示
{
  stroke(100, 100, 100 );
  strokeWeight( 2 );
  line( 22, 200, 22, 350 );
  line( 430, 200, 430, 350 );
  line( 22, 200, 430, 200 );
  line( 22, 350, 430, 350 );
  line( 22, 200+37.5*1, 430, 200+37.5*1 );
  line( 22, 200+37.5*2, 430, 200+37.5*2 );
  line( 22, 200+37.5*3, 430, 200+37.5*3 );
  line( 22+136*1, 200, 22+136*1, 350 );
  line( 22+136*2, 200, 22+136*2, 350 );  

  fill(255);
  textSize(18); 
  text("放射線量グラフ", 30, 225);
  textSize(12); 
  text("測定間隔(時間)はArduinoのスケッチで設定", 174, 215);
  text("Ｇキーで横軸倍率を〈１倍･10倍〉 切替", 174, 230);

  //右側目盛り
  textSize(12); 
  text("0.16", 435, 205);
  text("0.12", 435, 205+37.5*1);
  text("0.08", 435, 205+37.5*2);
  text("0.04", 435, 205+37.5*3);
  text("0.00", 435, 205+37.5*4);
  text("[μSv/h]", 415, 185);
}

//Sキー及びGキーに機能割り当て
void keyPressed() 
{
  if (key == 's'||key == 'S') {
    noStroke();
    fill(0);
    rect(245, 55, 200, 17);
    fill(255);
    textSize(14); 
    if (soundNo == 0) {
      soundNo = 1;
      text("放射線検出時の音：オン", 265, 70);
    } else if (soundNo ==1) {
      soundNo = 0;
      text("放射線検出時の音：オフ", 265, 70);
    }
  }
  if (key == 'g'||key == 'G') {
    if (graphNo == 0) {
      graphNo = 1;
      noStroke();
      fill(0);
      rect(0, 198, 465, 185);  
      graphA();
    } else if (graphNo ==1) {
      graphNo = 0;
      noStroke();
      fill(0);
      rect(0, 198, 465, 185); 
      graphB();
    }
  }
}

void stop() 
{
  song.close();
  minim.stop();
  super.stop();
}

