// Arduinoからmicro:bitへ書き込むスケッチ
//Radiation-Watch.orgのオリジナルスケッチを一部改変して作成した。（2019年8月　高木伸雄）
//////////////////////////////////////////////////
// Radiation-Watch.org
// URL http://www.radiation-watch.org/
//////////////////////////////////////////////////
//ここで測定値出力間隔を設定してください。秒単位です。
int interval = 60;

///　デジタルIOのピンの設定///
int signPin = 0; //micro:bitのpin0使用（信号）
int noisePin = 1; //micro:bitのpin1使用（ノイズ）
//VCC:micro:bitのpin3V使用(3.3V)
//GND:micro:bitのpinGND使用(GND)
//////////////////////////////////////////////////

const double alpha = 53.032; // cpm = uSv × alpha

int Index = 0; //ループを回った回数
char msg[256] = ""; //シリアルで出力する文字列用

int signCount = 0; //信号のカウント回数
int noiseCount = 0; //ノイズのカウント回数
//int signCountA = 0; //信号のカウント回数
int sON = 0; //信号検出時のロックフラグ
int nON = 0; //ノイズ検出時のロックフラグ

double cpm = 0; //現在のカウント数
double cpmHistory[200]; //カウント回数の履歴
int cpmIndex = 0; //カウント回数の現在配列の格納位置
int cpmIndexPrev = 0; //二重カウントの防止用

//ループの時間間隔の計測用
unsigned long prevTime = 0;
unsigned long currTime = 0;

int totalSec = 0; //計測開始からのトータルの時間(秒)
int interval_totalSec = 0; //測定値出力からのトータルの時間(秒)
int totalHour = 0; //計測開始からのトータルの時間(時間)

//cpm計算用の時間
int cpmTimeMSec = 0;
int cpmTimeSec = 0;

int signCountSum = 0;
int Count = 0;  //intervalの中の処理回数
float cpmBuffSum = 0;
float uSvBuffSum = 0;
float uSvdBuffSum = 0;

void setup()
{
  //シリアルの準備
  //9600bps
  Serial.begin(9600);
  pinMode(signPin, INPUT_PULLUP); //信号検出ピンの設定
  pinMode(noisePin, INPUT_PULLUP); //ノイズ検出ピンの設定

  //cpmの履歴を保存する配列の初期化
  for (int i = 0; i < 200; i++ )
  {
    cpmHistory[i] = 0;
  }

  //ループ開始時間を取得
  prevTime = millis();
}

void loop()
{
  // 信号のローデータ　通常:High(sign=1)  検出時:Low(sign=0)
  int sign = digitalRead(signPin);

  // ノイズのローデータ　通常:Low(noise=0)  検出時:High(noise=1)
  int noise = digitalRead(noisePin);

  //信号の検出(100us程度 Lowになる)
  if (sign == 0 && sON == 0)
  { //信号の検出開始、しばらく続くのでその間はカウントしないようにする
    sON = 1;
    signCount++;
    Serial.println("R"); //一文字をシリアルで送信し、モニター（PC）から音が出るようにする
  }
  else if (sign == 1 && sON == 1) {
    //信号が終了したのでフラグを解除
    sON = 0;
  }

  //ノイズの検出(100us程度 Lowになる)
  if (noise == 1 && nON == 0)
  { //ノイズの検出開始、しばらく続くのでその間はカウントしないようにする
    nON = 1;
    noiseCount++;
  }
  else if (noise == 0 && nON == 1) {
    //ノイズが終了したのでフラグを解除
    nON = 0;
  }

  //10000回ループを回ったら、計測値を計算して、シリアルで出力する
  if (Index == 10000)
  {
    //現在の時刻を取得
    currTime = millis();

    //ノイズが10000回のループの中でまったく検出されないときの処理
    //ノイズが検出されたときは、処理をしない
    if (noiseCount == 0)
    {
      //6秒ごとにカウント回数の履歴を格納する配列をずらす
      if ( totalSec % 6 == 0 && cpmIndexPrev != totalSec)
      {
        cpmIndexPrev = totalSec;
        cpmIndex++;

        //最後まできたら元に戻す
        if (cpmIndex >= 200)
        {
          cpmIndex = 0;
        }

        //一周(1200秒）してきたときに次に格納する配列に値が詰まっていれば、
        //現在の合計しているカウント(変数cpm)から引いて、常に最新の1200秒分の
        //カウント数にする。
        if (cpmHistory[cpmIndex] > 0)
        {
          cpm -= cpmHistory[cpmIndex];
        }
        cpmHistory[cpmIndex] = 0; //6秒ごとに0にして、カウント回数の履歴の蓄積に備える
      }

      //カウント回数の履歴を保存
      cpmHistory[cpmIndex] += signCount;
      //カウント回数の蓄積
      cpm += signCount;

      //10000回、データを取得するのにかかった時間を取得
      cpmTimeMSec += abs(currTime - prevTime);
      //ms→secに変換する(オーバーフロー対策)
      if (cpmTimeMSec >= 1000)
      {
        cpmTimeMSec -= 1000;
        //cpmを求めるときに使用する計測時間を加算する
        //計測時間が20分（1200秒）を過ぎると1200に固定する
        if ( cpmTimeSec >= 20 * 60 )
        {
          cpmTimeSec = 20 * 60;
        }
        else {
          cpmTimeSec++;
        }

        //トータルでの計測時間
        totalSec++;
        interval_totalSec++;
        //sec→hourに変換する(オーバーフロー対策)
        if (totalSec >= 3600)
        {
          totalSec -= 3600;
          totalHour++;
        }
      }

      //現在の計測時間(最大20分)
      double min = cpmTimeSec / 60.0;
      if (min != 0)
      {
        //cpm、uSv/h、uSv/hの誤差、をそれぞれ計算し、加算する
        signCountSum += signCount;
        cpmBuffSum += cpm / min;
        uSvBuffSum += cpm / min / alpha;
        uSvdBuffSum += sqrt(cpm) / min / alpha;
        Count ++ ;
      }

      //設定時間（interval）になったらデータ（平均値）を出力する
      if (interval_totalSec == interval) {
        //シリアルで送信
        Serial.print(totalHour);
        Serial.print(",");
        Serial.print(totalSec);
        Serial.print(",");
        Serial.print(signCountSum);
        Serial.print(",");

        if (min != 0) {
          Serial.print(cpmBuffSum / Count, 3); //cpmBuff
          Serial.print(",");
          Serial.print(uSvBuffSum / Count, 3); //uSvBuff
          Serial.print(",");
          Serial.print(uSvdBuffSum / Count, 3); //uSvdBuff
        } else {
          Serial.print(0.000, 3); //cpmBuff
          Serial.print(",");
          Serial.print(0.000, 3); //uSvBuff
          Serial.print(",");
          Serial.print(0.000, 3); //uSvdBuff
        }
        Serial.print(",");
        Serial.println(interval);
        signCountSum = 0;
        Count = 0;
        cpmBuffSum = 0;
        uSvBuffSum = 0;
        uSvdBuffSum = 0;
        interval_totalSec = 0;
      }
    }

    //次の10000回のループのための変数の初期化
    prevTime = currTime;
    signCount = 0;
    noiseCount = 0;
    Index = 0;
  }
  Index++;
}
